import React__default from 'react';
import { isFunction } from '@aws-amplify/ui';

// default state
const INITIAL_STATE = { hasError: false, isLoading: false, message: undefined };
const LOADING_STATE = { hasError: false, isLoading: true, message: undefined };
const ERROR_STATE = { hasError: true, isLoading: false };
const resolveMaybeAsync = async (value) => {
    const awaited = await value;
    return awaited;
};
/**
 * @internal may be updated in future versions
 */
function useDataState(action, initialData, options) {
    const [dataState, setDataState] = React__default.useState(() => ({
        ...INITIAL_STATE,
        data: initialData,
    }));
    const prevData = React__default.useRef(initialData);
    const pendingId = React__default.useRef();
    const { onSuccess, onError } = options ?? {};
    const handleAction = React__default.useCallback((input) => {
        const id = Symbol();
        pendingId.current = id;
        setDataState(({ data }) => ({ ...LOADING_STATE, data }));
        resolveMaybeAsync(action(prevData.current, input))
            .then((data) => {
            if (pendingId.current !== id)
                return;
            prevData.current = data;
            if (isFunction(onSuccess))
                onSuccess(data);
            setDataState({ ...INITIAL_STATE, data });
        })
            .catch((error) => {
            if (pendingId.current !== id)
                return;
            if (isFunction(onError))
                onError(error);
            const { message } = error;
            setDataState(({ data }) => ({ ...ERROR_STATE, data, message }));
        });
    }, [action, onError, onSuccess]);
    return [dataState, handleAction];
}

export { useDataState as default };
